SUBMERCADOS = ["SUDESTE", "SUL", "NORDESTE", "NORTE"]
NUM_CENARIOS = 2000
NUM_ANOS_ESTUDO = 5
NUM_PATAMARES = 3
NUM_VARIAVEIS_MEDIAS = 52
NUM_VARIAVEIS_CUSTO_PMO = 16
MESES = ["JANEIRO",
         "FEVEREIRO",
         "MARCO",
         "ABRIL",
         "MAIO",
         "JUNHO",
         "JULHO",
         "AGOSTO",
         "SETEMBRO",
         "OUTUBRO",
         "NOVEMBRO",
         "DEZEMBRO"]
